@@ -15,24 +15,7 @@
 //
 //--------------------------------------------------------------------------------
 
-#include "AppleUSBAudioCommon.h"
 #include "AppleUSBAudioDevice.h"
-#include "AppleUSBAudioLevelControl.h"
-#include "AppleUSBAudioMuteControl.h"
-#include "USBAudioObject.h"
-#include "AppleUSBAudioEngine.h"
-
-#include <IOKit/IOLib.h>
-#include <IOKit/IOKitKeys.h>
-#include <IOKit/IORegistryEntry.h>
-#include <IOKit/audio/IOAudioStream.h>
-#include <IOKit/audio/IOAudioDefines.h>
-
-#include <IOKit/usb/IOUSBInterface.h>
-#include <IOKit/usb/IOUSBRootHubDevice.h>
-#include <IOKit/IOMessage.h>
-
-#include <libkern/c++/OSCollectionIterator.h>
 
 #define super IOAudioDevice
 
@@ -98,7 +81,7 @@ void AppleUSBAudioDevice::free () {
 
     if (usbAudio) {
         usbAudio->release ();
-        usbAudio = 0;
+        usbAudio = NULL;
     }
 
     super::free ();
@@ -111,35 +94,70 @@ bool AppleUSBAudioDevice::requestTerminate (IOService * provider, IOOptionBits o
 	return TRUE;		// it is OK to terminate us
 }
 
+bool AppleUSBAudioDevice::ControlsStreamNumber (UInt8 streamNumber) {
+	UInt8 *							streamNumbers;
+	UInt8							numStreams;
+	UInt8							index;
+	bool							doesControl;
+
+	doesControl = FALSE;
+
+	if (usbAudio) {
+		usbAudio->GetControlledStreamNumbers (&streamNumbers, &numStreams);
+		for (index = 0; index < numStreams; index++) {
+			debug3IOLog ("Checking stream %d against controled stream %d\n", streamNumber, streamNumbers[index]);
+			if (streamNumber == streamNumbers[index]) {
+				doesControl = TRUE;
+				break;				// get out of for loop
+			}
+		}
+	}
+
+	return doesControl;
+}
+
 bool AppleUSBAudioDevice::start (IOService * provider) {
     OSObject *						obj;
 	UInt8							stringIndex;
 	char							string[kStringBufferSize];
 	IOReturn						err;
     Boolean							resultCode;
+	UInt8 *							streamNumbers;
+	UInt8							numStreams;
 
 	debug3IOLog ("+AppleUSBAudioDevice[%p]::start (%p)\n", this, provider);
 
 	resultCode = FALSE;
 
 	controlInterface = OSDynamicCast (IOUSBInterface, provider);
-	FailIf (NULL == controlInterface, Exit);
-	previousInterface = controlInterface;
+	FailIf (FALSE == controlInterface->open (this), Exit);
 
 	debug2IOLog ("There are %d configurations on this device\n", controlInterface->GetDevice()->GetNumConfigurations ());
-	usbAudio = USBAudioConfigObject::create (controlInterface->GetDevice()->GetFullConfigurationDescriptor (0));
+	debug2IOLog ("Our control interface number is %d\n", controlInterface->GetInterfaceNumber ());
+	usbAudio = USBAudioConfigObject::create (controlInterface->GetDevice()->GetFullConfigurationDescriptor (0), controlInterface->GetInterfaceNumber ());
 	FailIf (NULL == usbAudio, Exit);
 
+	// Check to make sure that the control interface we loaded against has audio streaming interfaces and not just MIDI.
+	usbAudio->GetControlledStreamNumbers (&streamNumbers, &numStreams);
+	debug2IOLog ("Num streams controlled = %d\n", numStreams);
+	debug2IOLog ("GetNumStreamInterfaces = %d\n", usbAudio->GetNumStreamInterfaces ());
+	FailIf (0 == numStreams, Exit);
+
 	// If this is an iSub, we need to not go any further because we don't support it in this driver
 	// This will cause the driver to not load on any device that has _only_ a low frequency effect output terminal
 	FailIf (usbAudio->GetNumOutputTerminals (0, 0) == 1 && usbAudio->GetIndexedOutputTerminalType (0, 0, 0) == OUTPUT_LOW_FREQUENCY_EFFECTS_SPEAKER, Exit);
 
 	err = kIOReturnError;
 	string[0] = 0;
+	stringIndex = controlInterface->GetInterfaceStringIndex ();
+	if (0 != stringIndex) {
+		err = controlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+	} else {
 	stringIndex = controlInterface->GetDevice()->GetProductStringIndex ();
 	if (0 != stringIndex) {
 		err = controlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
 	}
+	}
 
 	if (0 == string[0] || kIOReturnSuccess != err) {
 		strcpy (string, "Unknown USB Audio Device");
@@ -161,10 +179,6 @@ bool AppleUSBAudioDevice::start (IOService * provider) {
 	setManufacturerName (string);
 	setDeviceTransportType (kIOAudioDeviceTransportTypeUSB);
 
-	FailIf (FALSE == controlInterface->open (this), Exit);
-	// get the IOAudioDevice generic start routine
-	FailIf (FALSE == super::start (provider), Exit);
-
 	interfaceLock = IORecursiveLockAlloc ();
 	FailIf (NULL == interfaceLock, Exit);
 
@@ -193,6 +207,7 @@ bool AppleUSBAudioDevice::start (IOService * provider) {
 		interfaceNumber = obj;
 	}
 
+	FailIf (FALSE == super::start (provider), Exit);
 	resultCode = TRUE;
 
 Exit:
@@ -200,50 +215,6 @@ Exit:
 	return resultCode;
 }
 
-void AppleUSBAudioDevice::streamStopped (IOAudioEngine *audioEngine) {
-    // This must be called from within the workLoop
-    // The AppleUSBAudioEngine does so from _setState()
-    // which has the same requirement
-    debugIOLog ("+AppleUSBAudioDevice::streamStopped ()\n");
-    
-    if (allStreamsStopped()) {
-        terminate();
-    }
-    
-    debugIOLog ("-AppleUSBAudioDevice::streamStopped ()\n");
-}
-
-bool AppleUSBAudioDevice::allStreamsStopped () {
-    OSCollectionIterator *			audioEngineIterator;
-    IOAudioEngine *					audioEngine;
-    bool							result;
-
-    debug2IOLog ("+AppleUSBAudioDevice[%p]::allStreamsStopped ()\n", this);
-    result = TRUE;
-
-    FailIf (NULL == audioEngines, Exit);
-
-    // Need to do this on the IOWorkLoop
-    
-    audioEngineIterator = OSCollectionIterator::withCollection (audioEngines);
-
-    if (audioEngineIterator) {
-        while (audioEngine = (IOAudioEngine *)audioEngineIterator->getNextObject ()) {
-            if (OSDynamicCast (AppleUSBAudioEngine, audioEngine)) {	// We only care that the AppleUSBAudioEngines are stopped
-                if (audioEngine->getState () != kIOAudioEngineStopped) {
-                    result = FALSE;
-                    break;
-                }
-            }
-        }
-        audioEngineIterator->release ();
-    }
-
-Exit:
-    debug2IOLog ("-AppleUSBAudioDevice[%p]::allStreamsStopped ()\n", this);
-    return result;
-}
-
 IOReturn AppleUSBAudioDevice::performPowerStateChange (IOAudioDevicePowerState oldPowerState, IOAudioDevicePowerState newPowerState, UInt32 *microSecsUntilComplete) {
 	IOReturn						result;
 
@@ -456,7 +427,7 @@ IOReturn AppleUSBAudioDevice::message (UInt32 type, IOService * provider, void *
 	return kIOReturnSuccess;
 }
 
-IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, bool shouldStartAudioEngine, UInt8 interfaceNum, UInt8 altInterfaceNum) {
+IOReturn AppleUSBAudioDevice::createControlsForInterface (IOAudioEngine *audioEngine, UInt8 interfaceNum, UInt8 altInterfaceNum) {
     AppleUSBAudioEngine *				usbAudioEngine;
     UInt8								channelNum;
     AppleUSBAudioLevelControl *			speakerControl;
@@ -471,9 +442,10 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 	UInt8								controlInterfaceNum;
 	Boolean								shouldUpdatePRAM;
 
-    debug5IOLog ("+AppleUSBAudioDevice[%p]::activateAudioEngine (%p, %d) - rc=%d\n", this, audioEngine, shouldStartAudioEngine, getRetainCount());
+    debug4IOLog ("+AppleUSBAudioDevice[%p]::createControlsForInterface %d %d\n", this, interfaceNum, altInterfaceNum);
 
     result = kIOReturnError;
+	terminatingDriver = FALSE;
 	FailIf (NULL == controlInterface, Exit);
 
     usbAudioEngine = OSDynamicCast (AppleUSBAudioEngine, audioEngine);
@@ -498,7 +470,7 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 				break;	// get out of for loop
 			}
 		}
-		FailWithAction (0 == featureUnitID, result = kIOReturnSuccess, Finish);	// There isn't a feature unit connected to this input terminal
+		FailWithAction (0 == featureUnitID, result = kIOReturnSuccess, Exit);	// There isn't a feature unit connected to this input terminal
 		// The interface was opened in AppleUSBAudioEngine::initHardware
 		controlInterface->SetAlternateInterface (this, kRootAlternateSetting);
 
@@ -535,7 +507,7 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 					audioEngine->addDefaultAudioControl (speakerControl);
 					speakerControl->release ();
 				} else {
-					debug2IOLog ("++AppleUSBAudioDevice::activateAudioEngine () - error creating volume control for channelNum %d\n", channelNum);
+					debug2IOLog ("++AppleUSBAudioDevice::createControlsForInterface () - error creating volume control for channelNum %d\n", channelNum);
 				}
 			}
 		}
@@ -552,7 +524,7 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 					audioEngine->addDefaultAudioControl (muteControl);
 					muteControl->release ();
 				} else {
-					debug2IOLog ("++AppleUSBAudioDevice::activateAudioEngine () - error creating mute control for channelNum %d\n", channelNum);
+					debug2IOLog ("++AppleUSBAudioDevice::createControlsForInterface () - error creating mute control for channelNum %d\n", channelNum);
 				}
 			}
 		}
@@ -573,9 +545,10 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 				break;	// get out of for loop
 			}
 		}
-		FailWithAction (0 == featureUnitID, result = kIOReturnSuccess, Finish);	// There isn't a feature unit connected to this output terminal
+		FailWithAction (0 == featureUnitID, result = kIOReturnSuccess, Exit);	// There isn't a feature unit connected to this output terminal
 		// The interface was opened in AppleUSBAudioEngine::initHardware
 		controlInterface->SetAlternateInterface (this, kRootAlternateSetting);
+		debug2IOLog ("There are %d controls\n", usbAudio->GetNumChannels (interfaceNum, altInterfaceNum));
 		for (channelNum = 0; channelNum <= usbAudio->GetNumChannels (interfaceNum, altInterfaceNum); channelNum++) {
 			if (usbAudio->ChannelHasVolumeControl (controlInterfaceNum, 0, featureUnitID, channelNum)) {
 				micControl = AppleUSBAudioLevelControl::create (featureUnitID,
@@ -592,8 +565,10 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 					audioEngine->addDefaultAudioControl (micControl);
 					micControl->release ();
 				} else {
-					debug2IOLog ("++AppleUSBAudioDevice::activateAudioEngine () - error creating volume control for channelNum %d\n", channelNum);
+					debug2IOLog ("++AppleUSBAudioDevice::createControlsForInterface () - error creating volume control for channelNum %d\n", channelNum);
 				}
+			} else {
+				debug2IOLog ("channel %d doesn't have a volume control\n", channelNum);
 			}
 		}
 		muteControl = NULL;
@@ -609,18 +584,22 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 					audioEngine->addDefaultAudioControl (muteControl);
 					muteControl->release ();
 				} else {
-					debug2IOLog ("++AppleUSBAudioDevice::activateAudioEngine () - error creating mute control for channelNum %d\n", channelNum);
+					debug2IOLog ("++AppleUSBAudioDevice::createControlsForInterface () - error creating mute control for channelNum %d\n", channelNum);
 				}
 			}
 		}
     }
 
-Finish:
-    FailIf (kIOReturnSuccess != super::activateAudioEngine (audioEngine, shouldStartAudioEngine), Exit);
-	result = TRUE;
-
 Exit:
-	flushAudioControls ();
+	return result;
+}
+
+IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, bool shouldStartAudioEngine, UInt8 interfaceNum, UInt8 altInterfaceNum) {
+    IOReturn							result;
+
+    debug5IOLog ("+AppleUSBAudioDevice[%p]::activateAudioEngine (%p, %d) - rc=%d\n", this, audioEngine, shouldStartAudioEngine, getRetainCount());
+
+    result = super::activateAudioEngine (audioEngine, shouldStartAudioEngine);
 
     debug4IOLog ("-AppleUSBAudioDevice[%p]::activateAudioEngine (%p) - rc=%d\n", this, audioEngine, getRetainCount ());
     return result;
@@ -631,7 +610,7 @@ IOReturn AppleUSBAudioDevice::deviceRequest (IOUSBDevRequest *request, AppleUSBA
 
 	debug4IOLog ("+AppleUSBAudioDevice[%p]::deviceRequest (%p, %p)\n", self, request, completion);
 	result = kIOReturnSuccess;
-	if (self->controlInterface) {
+	if (self->controlInterface && FALSE == self->terminatingDriver) {
 		FailIf (NULL == self->interfaceLock, Exit);
 
         IORecursiveLockLock (self->interfaceLock);
@@ -645,6 +624,19 @@ Exit:
 	return result;
 }
 
+bool AppleUSBAudioDevice::willTerminate (IOService * provider, IOOptionBits options) {
+	debug3IOLog ("+AppleUSBAudioDevice[%p]::willTerminate (%p)\n", this, provider);
+
+	if (controlInterface == provider) {
+		terminatingDriver = TRUE;
+	}
+
+	debug2IOLog ("-AppleUSBAudioDevice[%p]::willTerminate\n", this);
+
+	return super::willTerminate (provider, options);
+}
+
+
 #ifdef DEBUG
 
 void AppleUSBAudioDevice::retain() const
@@ -656,7 +648,7 @@ void AppleUSBAudioDevice::retain() const
 void AppleUSBAudioDevice::release() const
 {
 //    debug3IOLog("AppleUSBAudioDevice(%p)::release() - rc=%d\n", this, getRetainCount());
-    super::release();
+	super::release();
 }
 
 #endif
