@@ -11,18 +11,31 @@
 #ifndef _APPLEUSBAUDIODEVICE_H
 #define _APPLEUSBAUDIODEVICE_H
 
+#include <libkern/c++/OSCollectionIterator.h>
+
+#include <IOKit/IOLocks.h>
+#include <IOKit/IOLib.h>
+#include <IOKit/IOKitKeys.h>
+#include <IOKit/IORegistryEntry.h>
+#include <IOKit/IOMessage.h>
+
 #include <IOKit/audio/IOAudioDevice.h>
 #include <IOKit/audio/IOAudioEngine.h>
 #include <IOKit/audio/IOAudioPort.h>
 #include <IOKit/audio/IOAudioControl.h>
+#include <IOKit/audio/IOAudioStream.h>
+#include <IOKit/audio/IOAudioDefines.h>
 
+#include <IOKit/usb/usb.h>
+#include <IOKit/usb/IOUSBInterface.h>
+#include <IOKit/usb/IOUSBRootHubDevice.h>
+
+#include "AppleUSBAudioMuteControl.h"
+#include "AppleUSBAudioLevelControl.h"
+#include "AppleUSBAudioEngine.h"
 #include "AppleUSBAudioCommon.h"
 #include "USBAudioObject.h"
 
-#include <IOKit/usb/IOUSBInterface.h>
-
-#include <IOKit/IOLocks.h>
-
 #define kStringBufferSize				255
 
 enum {
@@ -51,18 +64,21 @@ class AppleUSBAudioDevice : public IOAudioDevice {
     OSDeclareDefaultStructors (AppleUSBAudioDevice);
 
 public:
-    IOUSBInterface *		controlInterface;
+    IOUSBInterface *					controlInterface;
 
 protected:
-    USBAudioConfigObject *	usbAudio;
+    USBAudioConfigObject *				usbAudio;
     IOUSBInterface *		previousInterface;
-    IORecursiveLock *		interfaceLock;
+    IORecursiveLock *					interfaceLock;
     OSObject *				interfaceVendor;
     OSObject *				interfaceProduct;
     OSObject *				deviceReleaseNumber;
     OSObject *				configurationValue;
     OSObject *				interfaceNumber;
     
+	Boolean								terminatingDriver;
+	thread_call_t						mInitHardwareThread;
+
 public:
     virtual bool init (OSDictionary *properties);
     virtual void free ();
@@ -75,16 +91,17 @@ public:
     virtual bool terminate (IOOptionBits options = 0);
     virtual bool finalize(IOOptionBits options);
     virtual IOReturn message (UInt32 type, IOService * provider, void * arg);
+	virtual bool ControlsStreamNumber (UInt8 streamNumber);
+	virtual IOReturn		createControlsForInterface (IOAudioEngine *audioEngine, UInt8 interfaceNum, UInt8 altInterfaceNum);
     virtual IOReturn activateAudioEngine (IOAudioEngine *audioEngine, bool shouldStartAudioEngine, UInt8 interfaceNum, UInt8 altInterfaceNum);
 	static	IOReturn deviceRequest (IOUSBDevRequest * request, AppleUSBAudioDevice * self, IOUSBCompletion * completion = 0);
-    virtual bool allStreamsStopped ();
 	virtual IOReturn performPowerStateChange (IOAudioDevicePowerState oldPowerState, IOAudioDevicePowerState newPowerState, UInt32 *microSecsUntilComplete);
     virtual USBAudioConfigObject * GetUSBAudioConfigObject (void) {return usbAudio;}
-    virtual void streamStopped (IOAudioEngine *audioEngine);
+	virtual bool			willTerminate (IOService * provider, IOOptionBits options);
 
 #ifdef DEBUG
-    virtual void retain() const;
-    virtual void release() const;
+    virtual void			retain() const;
+    virtual void			release() const;
 #endif
 };
 
