@@ -11,15 +11,38 @@
 #ifndef _APPLEUSBAUDIOENGINE_H
 #define _APPLEUSBAUDIOENGINE_H
 
-#define NEWUSB					TRUE
+#include <libkern/OSByteOrder.h>
+#include <libkern/c++/OSCollectionIterator.h>
+#include <libkern/c++/OSMetaClass.h>
 
+#include <IOKit/IOLib.h>
+#include <IOKit/IOSyncer.h>
+#include <IOKit/IOService.h>
+#include <IOKit/IOMessage.h>
+#include <IOKit/IOMemoryCursor.h>
+#include <IOKit/IOMemoryDescriptor.h>
 #include <IOKit/IOBufferMemoryDescriptor.h>
+#include <IOKit/IOTimerEventSource.h>
+#include <IOKit/IOMultiMemoryDescriptor.h>
+
+#include <IOKit/audio/IOAudioDevice.h>
+#include <IOKit/audio/IOAudioPort.h>
+#include <IOKit/audio/IOAudioTypes.h>
+#include <IOKit/audio/IOAudioDefines.h>
+#include <IOKit/audio/IOAudioLevelControl.h>
+//#include <IOKit/audio/IOAudioToggleControl.h>
 #include <IOKit/audio/IOAudioEngine.h>
 #include <IOKit/audio/IOAudioStream.h>
+
 #include <IOKit/usb/IOUSBPipe.h>
 #include <IOKit/usb/IOUSBDevice.h>
 #include <IOKit/usb/IOUSBInterface.h>
 
+#include "AppleUSBAudioCommon.h"
+#include "AppleUSBAudioDevice.h"
+#include "AppleUSBAudioLevelControl.h"
+#include "AppleUSBAudioMuteControl.h"
+
 #include "USBAudioObject.h"
 #include "AppleiSubEngine.h"
 #include "AppleUSBAudioClip.h"
@@ -36,7 +59,6 @@ class AppleUSBAudioDevice;
 
 #define kMinimumFrameOffset				1
 
-
 #define RECORD_NUM_USB_FRAME_LISTS				128
 #define RECORD_NUM_USB_FRAMES_PER_LIST			2
 #define RECORD_NUM_USB_FRAME_LISTS_TO_QUEUE		64
@@ -71,119 +93,96 @@ public:
     virtual IOReturn performAudioEngineStart ();
     virtual IOReturn performAudioEngineStop ();
     
-    virtual IOAudioEngineState _setState (IOAudioEngineState newState);
-
 	static void sampleRateHandler (void * target, void * parameter, IOReturn result, IOUSBIsocFrame * pFrames);
 
-#if NEWUSB
     static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
     static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
-#else
-    static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBIsocFrame * pFrames);
-    static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBIsocFrame * pFrames);
-#endif
-
 	static IOReturn iSubAttachChangeHandler (IOService *target, IOAudioControl *passThruControl, SInt32 oldValue, SInt32 newValue);
 
 protected:
 	UInt64								usbFrameToQueueAt;
-	UInt64					nextSynchReadFrame;
-#if NEWUSB
+	UInt64								nextSynchReadFrame;
 	IOUSBLowLatencyIsocFrame *			theUSBIsocFrames;
 	IOUSBLowLatencyIsocFrame			theStartFrame;
-#else
-	IOUSBIsocFrame *					theUSBIsocFrames;
-	IOUSBIsocFrame						theStartFrame;
-#endif
-	IOUSBIsocFrame			theSampleRateFrame;
-#if NEWUSB
+	IOUSBIsocFrame						theSampleRateFrame;
 	IOUSBLowLatencyIsocCompletion *		usbCompletion;
-#else
-	IOUSBIsocCompletion *	usbCompletion;
-#endif
-	IOUSBIsocCompletion		sampleRateCompletion;
-	IOUSBInterface *		streamInterface;
-	IOUSBInterface *		previousInterface;
-	IOUSBPipe *				thePipe;
-	IOUSBPipe *				theAssociatedPipe;
+	IOUSBIsocCompletion					sampleRateCompletion;
+	IOUSBInterface *					streamInterface;
+	IOUSBInterface *					previousInterface;
+	IOUSBPipe *							thePipe;
+	IOUSBPipe *							theAssociatedPipe;
 	IOSyncer *							signal;
 	AppleUSBAudioDevice *				usbAudioDevice;
 	IOMemoryDescriptor **				soundBufferDescriptors;
-	IOMemoryDescriptor *	theStartBufferDescriptor;
-	IOMemoryDescriptor *	neededSampleRate;
+	IOMemoryDescriptor *				theStartBufferDescriptor;
+	IOMemoryDescriptor *				neededSampleRate;
 	IOVirtualRange						theWrapRanges[2];
-	IONotifier *			streamNotifier;
-	OSObject *				interfaceVendor;
-	OSObject *				interfaceProduct;
-	OSObject *				deviceReleaseNumber;
-	OSObject *				configurationValue;
-	OSObject *				interfaceNumber;
-	void *					readBuffer;
-	void *					theStartBuffer;
+	IONotifier *						streamNotifier;
+	OSObject *							interfaceVendor;
+	OSObject *							interfaceProduct;
+	OSObject *							deviceReleaseNumber;
+	OSObject *							configurationValue;
+	OSObject *							interfaceNumber;
+	void *								readBuffer;
+	void *								theStartBuffer;
 	UInt32 *							aveSampleRateBuf;		// only needs to be 3 bytes for the 10.14 value
-#if !NEWUSB
-	UInt64								expectedBusFrame;		// For output, the frame on which we expect to get our next USB isoc completion
-																// For input, the frame on which we last received a USB isoc completion and
-																// wrapped the sample buffer.
-#endif
-	thread_call_t			iSubTeardownThreadCall;
-	thread_call_t			iSubOpenThreadCall;
-	IOAudioSampleRate		curSampleRate;
+	thread_call_t						iSubTeardownThreadCall;
+	thread_call_t						iSubOpenThreadCall;
+	IOAudioSampleRate					curSampleRate;
 	UInt32								previouslyPreparedBufferOffset;
 	UInt32								safeToEraseTo;
 	UInt32								lastSafeErasePoint;
-	UInt32					theStartBufferSize;
-	UInt32					previous;
-	UInt32					previousClippedToFrame;
-	UInt32					theMaxPacket;
+	UInt32								theStartBufferSize;
+	UInt32								previous;
+	UInt32								previousClippedToFrame;
 	UInt32								readUSBFrameListSize;
-	UInt32					bufferOffset;
-	UInt32					averageSampleRate;
-	UInt32					sum;					// This is the number of samples we have sent in the previous frames in this period
+	UInt32								bufferOffset;
+	UInt32								averageSampleRate;
+	UInt32								sum;					// This is the number of samples we have sent in the previous frames in this period
 	UInt32								numUSBFrameLists;
 	UInt32								numUSBFramesPerList;
 	UInt32								numUSBFrameListsToQueue;
-	UInt32					currentFrameList;
-	UInt32					bufferSize;
-	UInt32					bytesPerSampleFrame;
-	UInt32					safetyOffset;
+	UInt32								currentFrameList;
+	UInt32								bufferSize;
+	UInt32								bytesPerSampleFrame;
+	UInt32								safetyOffset;
 	UInt32								fractionalSamplesRemaining;
-	UInt16					samplesThisFrame;
-	UInt8					ourInterfaceNumber;
-	UInt8					shouldStop;
-	UInt8					alternateInterfaceID;
-	UInt8					refreshInterval;
-	UInt8					framesUntilRefresh;
-	UInt8					direction;
+	UInt16								samplesThisFrame;
+	UInt8								ourInterfaceNumber;
+	UInt8								shouldStop;
+	UInt8								alternateInterfaceID;
+	UInt8								refreshInterval;
+	UInt8								framesUntilRefresh;
+	UInt8								direction;
 	Boolean								inCompletion;
-	Boolean					usbStreamRunning;
-	Boolean					notifyDeviceOfStop;
-	Boolean					startingEngine;
-	Boolean					startiSub;
-	Boolean					justResetClipPosition;		// aml 3.29.02 added to match AOA
-	Boolean					gotRequestClose;
-	Boolean					gotStopCall;
-	Boolean					iSubIsOpen;
-	Boolean					terminatingDriver;
-	IOAudioStream *			mainStream;
+	Boolean								usbStreamRunning;
+	Boolean								notifyDeviceOfStop;
+	Boolean								startingEngine;
+	Boolean								startiSub;
+	Boolean								justResetClipPosition;		// aml 3.29.02 added to match AOA
+	Boolean								gotRequestClose;
+	Boolean								gotStopCall;
+	Boolean								iSubIsOpen;
+	Boolean								terminatingDriver;
+	IOAudioStream *						mainStream;
 
 	// For iSub
-	IOMemoryDescriptor *	iSubBufferMemory;
-	UInt32					iSubLoopCount;
-	SInt32					iSubBufferOffset;		// Where we are writing to in the iSub buffer
-	IOService *				ourProvider;
-	IONotifier *			iSubEngineNotifier;
-	IOAudioToggleControl *	iSubAttach;
-	AppleiSubEngine *		iSubEngine;
-	AppleiSubEngine *		oldiSubEngine;
-	float *					lowFreqSamples;
-	float *					highFreqSamples;
-	PreviousValues				filterState;
-	PreviousValues				filterState2;
-	PreviousValues				phaseCompState;
-	float					srcPhase;			 	
-	float					srcState;			 	
-	Boolean					needToSync;
+	IOMemoryDescriptor *				iSubBufferMemory;
+	UInt32								iSubLoopCount;
+	SInt32								iSubBufferOffset;		// Where we are writing to in the iSub buffer
+	IOService *							ourProvider;
+	IONotifier *						iSubEngineNotifier;
+	IOAudioToggleControl *				iSubAttach;
+	AppleiSubEngine *					iSubEngine;
+	AppleiSubEngine *					oldiSubEngine;
+	float *								lowFreqSamples;
+	float *								highFreqSamples;
+	PreviousValues						filterState;
+	PreviousValues						filterState2;
+	PreviousValues						phaseCompState;
+	float								srcPhase;
+	float								srcState;
+	Boolean								needToSync;
 
 	static inline IOFixed IOUFixedDivide(UInt32 a, UInt32 b)
 	{
@@ -225,13 +224,7 @@ protected:
     virtual IOAudioStreamDirection getDirection (void);
     virtual void *getSampleBuffer (void);
 	virtual UInt32 getSampleBufferSize (void);
-
-#if NEWUSB
 	virtual void CoalesceInputSamples (UInt32 numBytesToCoalesce, IOUSBLowLatencyIsocFrame * pFrames);
-#else
-	virtual void CoalesceInputSamples (UInt32 numBytesToCoalesce, IOUSBIsocFrame * pFrames);
-#endif
-
 	virtual void resetClipPosition (IOAudioStream *audioStream, UInt32 clipSampleFrame);
     virtual IOReturn clipOutputSamples (const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
 	virtual IOReturn convertInputSamples (const void *sampleBuf, void *destBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
