@@ -1,21 +1,4 @@
-#include <sys/cdefs.h>
-
-__BEGIN_DECLS
-#include <kern/thread_call.h>
-__END_DECLS
-
 #include "AppleUSBAudioLevelControl.h"
-#include "AppleUSBAudioCommon.h"
-#include "AppleUSBAudioDevice.h"
-
-#include <IOKit/audio/IOAudioTypes.h>
-#include <IOKit/audio/IOAudioDefines.h>
-#include <IOKit/usb/USB.h>
-#include <IOKit/usb/IOUSBInterface.h>
-#include <IOKit/IOLib.h>
-#include <IOKit/IOPlatformExpert.h>
-
-#include <libkern/OSByteOrder.h>
 
 #define super IOAudioLevelControl
 
@@ -124,8 +107,17 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 	debug2IOLog ("offset = %d\n", offset);
 
 	currentValue = (currentValue + offset) / volRes;
+	if (deviceMin < 0 && deviceMax > 0) {
+		deviceMax += volRes;
+		debug2IOLog ("deviceMax adjusted to = 0x%X\n", deviceMax);
+	}
 	deviceMax = ((deviceMin + offset) + (deviceMax + offset)) / volRes;
 
+	if (kIOAudioLevelControlSubTypeLFEVolume == subType) {
+		currentValue = currentValue / 2;
+		updateUSBValue (currentValue);
+	}
+
 	// Set values needed to compute proper PRAM boot beep volume setting
 	fMaxVolume = deviceMax;
 	fMinVolume = deviceMin + offset;
@@ -216,10 +208,15 @@ void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
 	if (newValue < 0) {
 		newVolume = 0x8000;
 	} else {
-		newVolume = (newValue * volRes) - offset;
+		if (newValue > 0) {
+			newVolume = ((newValue - 1) * volRes) - offset;
+		} else {
+			newVolume = (newValue * volRes) - offset;
+		}
 	}
+	debug2IOLog ("volume value is 0x%X\n", newVolume);
     theValue = HostToUSBWord (newVolume);
-	debug2IOLog ("setting volume to 0x%X\n", newVolume);
+	debug2IOLog ("setting volume to 0x%X (little endian)\n", newVolume);
 	ret = SetCurVolume (interfaceNumber, channelNumber, theValue);
 
 	if (getSubType () == kIOAudioLevelControlSubTypeLFEVolume) {
@@ -230,36 +227,7 @@ void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
 	if (TRUE == fShouldUpdatePRAM && FALSE == gExpertMode) {				// We do that only if we are on a OS 9 like UI guideline
 		WritePRAMVol (newValue, newValue);
 	}
-#if 0
-	// If this is the iSub volume control, don't bother to look for ourselves, which would probably result in an infinite loop anyway.
-	if (getSubType () != kIOAudioLevelControlSubTypeLFEVolume && getUsage () == kIOAudioControlUsageOutput) {
-		IOAudioLevelControl *	iSubVolume;
-		IORegistryEntry *		start;
-		IORegistryEntry *		parent;
-		IORegistryEntry *		engine;
 
-		iSubVolume = NULL;
-		debugIOLog ("Looking for iSub volume control\n");
-		start = getParentEntry (gIOServicePlane);
-		FailIf (NULL == start, Exit);
-		parent = start->getParentEntry (gIOServicePlane);
-		FailIf (NULL == parent, Exit);
-		engine = parent->childFromPath ("AppleUSBAudioEngine", gIOServicePlane);
-		FailIf (NULL == engine, Exit);
-
-		iSubVolume = (IOAudioLevelControl *)FindEntryByNameAndProperty (engine, "AppleUSBAudioLevelControl", kIOAudioControlSubTypeKey, kIOAudioLevelControlSubTypeLFEVolume);
-
-		engine->release ();
-
-		if (NULL != iSubVolume) {
-			newiSubVolume = ((newValue * kiSubMaxVolume) / getMaxValue ()) * kiSubVolumePercent / 100;
-			debugIOLog ("Setting the iSub volume control\n");
-			iSubVolume->setValue (newiSubVolume);
-			iSubVolume->release ();
-		}
-	}
-#endif
-//Exit:
     if (ret != kIOReturnSuccess) {
         debug4IOLog ("++AppleUSBAudioLevelContol:updateUSBValue () - set current value for %d:%d failed: 0x%X\n", controlSelector, channelNumber, ret);
     }
@@ -422,37 +390,7 @@ IOFixed AppleUSBAudioLevelControl::ConvertUSBVolumeTodB (SInt16 volume) {
 
 	return dBVolumeFixed;
 }
-#if 0
-IORegistryEntry * AppleUSBAudioLevelControl::FindEntryByNameAndProperty (const IORegistryEntry * start, const char * name, const char * key, UInt32 value) {
-	OSIterator				*iterator;
-	IORegistryEntry			*theEntry;
-	IORegistryEntry			*tmpReg;
-	OSNumber				*tmpNumber;
 
-	theEntry = NULL;
-	iterator = NULL;
-	FailIf (NULL == start, Exit);
-
-	iterator = start->getChildIterator (gIOServicePlane);
-	FailIf (NULL == iterator, Exit);
-
-	while (NULL == theEntry && (tmpReg = OSDynamicCast (IORegistryEntry, iterator->getNextObject ())) != NULL) {
-		if (strcmp (tmpReg->getName (), name) == 0) {
-			tmpNumber = OSDynamicCast (OSNumber, tmpReg->getProperty (key));
-			if (NULL != tmpNumber && tmpNumber->unsigned32BitValue () == value) {
-				theEntry = tmpReg;
-				theEntry->retain ();
-			}
-		}
-	}
-
-Exit:
-	if (NULL != iterator) {
-		iterator->release ();
-	}
-	return theEntry;
-}
-#endif
 #pragma mark +PRAM VOLUME
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //	Calculates the PRAM volume value for stereo volume.
