@@ -5,65 +5,68 @@
 #include "USBAudioObject.h"
 
 #include <IOKit/audio/IOAudioStream.h>
-#include <IOKit/audio/IOAudioEngine.h> 
+#include <IOKit/audio/IOAudioEngine.h>
+#include <IOKit/pwr_mgt/RootDomain.h>
 
 #define kiSubFeatureUnitID					2
 #define kiSubMuteControlChannelNum			0
-#define kiSubVolumeControlLeftChannelNum	1
-#define kiSubVolumeControlRightChannelNum	2
+#define kiSubVolumeControlMasterChannelNum	1
 
 // Has to be true or else the iSub looses the beginning of sounds if it has auto powered down
-#define CONTINUOUS_STREAMING				TRUE
-#define DEBUGTIMESTAMPS						FALSE
-#define DEBUGLOG							FALSE
+#define NUM_POWER_STATES					2
 
 #define super IOService
 
+// aml 02.14.02 mandatory definitions of static constants
+const UInt32 AppleiSubEngine::kDefaultOutputSampleRate;		
+const UInt32 AppleiSubEngine::kDefaultNumChannels;		
+const UInt32 AppleiSubEngine::kDefaultBytesPerSample;		
+const iSubAltInterfaceType AppleiSubEngine::kDefaultiSubAltInterface;		
+
 OSDefineMetaClassAndStructors (AppleiSubEngine, super)
 
 #pragma mark -IOKit Routines-
 
-// This has to be called on the audio engine's work loop so that we don't interrupt the IOAudioFamily
-void AppleiSubEngine::close (IOService * forClient, IOOptionBits options) {
-	IOReturn			result;
+void AppleiSubEngine::closeiSub (IOService * forClient) {
+	IOReturn				result;
 
-	debug4IOLog ("+AppleiSubEngine[%p]::close (%p, 0x%lx)\n", this, forClient, options);
+	debug3IOLog ("+AppleiSubEngine[%p]::closeiSub (%p)\n", this, forClient);
 
-	if (NULL != audioEngine) {
-		debugIOLog ("removing iSub audio controls\n");
-//		audioEngine->pauseAudioEngine ();
-		audioEngine->beginConfigurationChange ();
-		result = audioEngine->removeDefaultAudioControl (leftVolumeControl);
+	if (forClient == audioEngine) {
+		if ((NULL != muteControl || NULL != iSubVolumeControl) && NULL != audioEngine) {
+			debugIOLog ("Removing iSub audio controls\n");
+	
+			if (NULL != iSubVolumeControl) {
+				result = audioEngine->removeDefaultAudioControl (iSubVolumeControl);
+				iSubVolumeControl = NULL;
+				#if DEBUGLOG
+				if (kIOReturnSuccess != result) IOLog ("Error 0x%x removing left iSub control\n", result);
+				#endif
+			}
+	
+			if (NULL != muteControl) {
+				result = audioEngine->removeDefaultAudioControl (muteControl);
+				muteControl = NULL;
+				#if DEBUGLOG
+				if (kIOReturnSuccess != result) IOLog ("Error 0x%x removing mute iSub control\n", result);
+				#endif
+			}
+	
+			release ();		// It's safe to release us now, there are no more controls that might be able to call into us.
+			// Don't try to use our audio engine host anymore
+			audioEngine = NULL;
 #if DEBUGLOG
-		if (kIOReturnSuccess != result) IOLog ("error 0x%x removing left iSub control\n", result);
+		} else {
+			if (NULL == muteControl) {IOLog ("NULL == muteControl\n");}
+			if (NULL == iSubVolumeControl) {IOLog ("NULL == iSubVolumeControl\n");}
+			if (NULL == audioEngine) {IOLog ("NULL == audioEngine\n");}
 #endif
-		result = audioEngine->removeDefaultAudioControl (rightVolumeControl);
-#if DEBUGLOG
-		if (kIOReturnSuccess != result) IOLog ("error 0x%x removing right iSub control\n", result);
-#endif
-		result = audioEngine->removeDefaultAudioControl (muteControl);
-#if DEBUGLOG
-		if (kIOReturnSuccess != result) IOLog ("error 0x%x removing mute iSub control\n", result);
-#endif
-		audioEngine->completeConfigurationChange ();
-//		audioEngine->resumeAudioEngine ();
-		audioEngine = NULL;
+		}
+	} else {
+		debugIOLog ("the wrong client tried to close us\n");
 	}
 
-	super::close (forClient, options);
-
-	debug4IOLog ("-AppleiSubEngine[%p]::close (%p, 0x%lx)\n", this, forClient, options);
-}
-
-bool AppleiSubEngine::finalize (IOOptionBits options) {
-	Boolean							resultCode;
-
-	debug4IOLog ("+AppleiSubEngine[%p]::finalize (%p), rc=%d\n", this, options, getRetainCount ());
-
-	resultCode = super::finalize (options);
-
-	debug5IOLog ("-AppleiSubEngine[%p]::finalize (%p) = %d, rc=%d\n", this, options, resultCode, getRetainCount ());
-	return resultCode;
+	debug3IOLog ("-AppleiSubEngine[%p]::closeiSub (%p)\n", this, forClient);
 }
 
 void AppleiSubEngine::free (void) {
@@ -93,8 +96,6 @@ void AppleiSubEngine::free (void) {
 		}
 	}
 
-        IORecursiveLockFree (interfaceLock);
-
 	super::free ();
 
 	debug2IOLog ("-AppleiSubEngine[%p]::free ()\n", this);
@@ -102,114 +103,39 @@ void AppleiSubEngine::free (void) {
 }
 
 // This has to be called on the audio engine's work loop so that we don't interrupt the IOAudioFamily
-bool AppleiSubEngine::handleOpen (IOService * forClient, IOOptionBits options, void * arg) {
-	IOUSBFindInterfaceRequest		findRequest;
-	IOUSBFindEndpointRequest		audioIsochEndpoint;
-	IOReturn						resultIOReturn;
-	UInt32							i;
-#if CONTINUOUS_STREAMING
-    AbsoluteTime					currentTime;
-	AbsoluteTime					initialTimestampOffset;
-	UInt32							frameListNum;
-	UInt16							numQueued;
-#endif
-	bool							resultCode;
-
-	debug5IOLog ("+AppleiSubEngine[%p]::handleOpen (%p, 0x%lx, %p)\n", this, forClient, options, arg);
+bool AppleiSubEngine::openiSub (IOService * forClient) {
+    bool						resultCode;
 
 	resultCode = FALSE;
 
-	FailIf (FALSE == super::handleOpen (forClient, options, arg), Exit);
-	FailIf (NULL == forClient, Exit);
-	audioEngine = (IOAudioEngine *)forClient;
+	// Only one client at a time!
+	FailIf (NULL != audioEngine, Exit);
+	audioEngine = OSDynamicCast (IOAudioEngine, forClient);
+	FailIf (NULL == audioEngine, Exit);
 
-	frameListSize = CalculateNumSamplesPerBuffer (44100, NUM_ISUB_FRAMES_PER_LIST) * 4;
-	debug2IOLog ("frameListSize = %d\n", frameListSize);
-	bufferSize = CalculateNumSamplesPerBuffer (44100, NUM_ISUB_FRAMES_PER_LIST, NUM_ISUB_FRAME_LISTS) * 4;
-	debug2IOLog ("bufferSize = %d\n", bufferSize);
-	sampleBuffer = IOMallocAligned (round_page (bufferSize), PAGE_SIZE);
-	FailIf (NULL == sampleBuffer, Exit);
-	for (i = 0; i < bufferSize / 4; i++) {
-		((UInt32*)sampleBuffer)[i] = 0;
-	}
-	sampleBufferDescriptor = IOMemoryDescriptor::withAddress (sampleBuffer, bufferSize, kIODirectionNone);
-	FailIf (NULL == sampleBufferDescriptor, Exit);
-
-	FailIf (kIOReturnSuccess != PrepareFrameLists (frameListSize), Exit);
-	FailIf (FALSE == streamInterface->open (this), Exit);
-
-	resultIOReturn = streamInterface->SetAlternateInterface (this, kRootAlternateSetting);
-	FailIf (kIOReturnSuccess != resultIOReturn, Exit);
-
-	resultIOReturn = streamInterface->SetAlternateInterface (this, 4);		// This is the 16-bit stereo interface
-	FailIf (kIOReturnSuccess != resultIOReturn, Exit);
-
-	// Acquire a PIPE for the isochronous stream.
-	audioIsochEndpoint.type = kUSBIsoc;
-	audioIsochEndpoint.direction = kUSBOut;
-
-	thePipe = streamInterface->FindNextPipe (NULL, &audioIsochEndpoint);
-	FailIf (NULL == thePipe, Exit);
-	thePipe->retain ();
-
-#if CONTINUOUS_STREAMING
-	// Start the iSub playing silence now
-	loopCount = 0;
-	nanoseconds_to_absolutetime ((kMinimumiSubFrameOffset) * 1000 * 1000, &initialTimestampOffset);
-	clock_get_uptime (&currentTime);
-	ADD_ABSOLUTETIME (&currentTime, &initialTimestampOffset);
-	lastLoopTime.hi = currentTime.hi;
-	lastLoopTime.lo = currentTime.lo;
-
-	shouldStop = 0;
-	numQueued = 0;
-	currentFrameList = 0;
-	currentByteOffset = 0;
-	theFirstFrame = streamInterface->GetDevice()->GetBus()->GetFrameNumber () + kMinimumiSubFrameOffset;
-	for (frameListNum = currentFrameList; numQueued < NUM_ISUB_FRAME_LISTS_TO_QUEUE; frameListNum++) {
-		resultIOReturn = WriteFrameList (frameListNum);
-		FailIf (kIOReturnSuccess != resultIOReturn, Exit);
-		numQueued++;
-	}
-#endif
-
-	// Find control interface on iSub device
-	findRequest.bInterfaceClass = 1;
-	findRequest.bInterfaceSubClass = 1;
-	findRequest.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
-	findRequest.bAlternateSetting = kIOUSBFindInterfaceDontCare;
-	controlInterface = streamInterface->GetDevice()->FindNextInterface (NULL, &findRequest);
-	FailIf (NULL == controlInterface, Exit);
+//	audioEngine->joinPMtree (this);
+//	if (pm_vars != NULL) {
+//		registerPowerDriver (this, iSubPowerStates, NUM_POWER_STATES);
+//		changePowerStateTo (1);
+//	}
 
 	audioEngine->pauseAudioEngine ();
 	audioEngine->beginConfigurationChange ();
-	leftVolumeControl = AppleUSBAudioLevelControl::create (kiSubFeatureUnitID,
+	retain ();		// As long as there are controls out there, don't let ourselves be released.
+
+	// Only need a master volume control now that the iSub is running mono.
+	iSubVolumeControl = AppleUSBAudioLevelControl::create (kiSubFeatureUnitID,
 													controlInterface->GetInterfaceNumber (),
 													VOLUME_CONTROL,
-													kiSubVolumeControlLeftChannelNum,
+													kiSubVolumeControlMasterChannelNum,
 													FALSE,
 													(USBDeviceRequest)&deviceRequest,
 													this,
-													'subL',
+													kIOAudioLevelControlSubTypeLFEVolume,
 													kIOAudioControlUsageOutput);
-	FailIf (NULL == leftVolumeControl, Exit);
+	FailIf (NULL == iSubVolumeControl, Exit);
 
-	audioEngine->addDefaultAudioControl (leftVolumeControl);
-	leftVolumeControl->release ();
-
-	rightVolumeControl = AppleUSBAudioLevelControl::create (kiSubFeatureUnitID,
-													controlInterface->GetInterfaceNumber (),
-													VOLUME_CONTROL,
-													kiSubVolumeControlRightChannelNum,
-													FALSE,
-													(USBDeviceRequest)&deviceRequest,
-													this,
-													'subR',
-													kIOAudioControlUsageOutput);
-	FailIf (NULL == rightVolumeControl, Exit);
-
-	audioEngine->addDefaultAudioControl (rightVolumeControl);
-	rightVolumeControl->release ();
+	audioEngine->addDefaultAudioControl (iSubVolumeControl);
 
 	muteControl = AppleUSBAudioMuteControl::create (kiSubFeatureUnitID,
 													controlInterface->GetInterfaceNumber (),
@@ -217,11 +143,10 @@ bool AppleiSubEngine::handleOpen (IOService * forClient, IOOptionBits options, v
 													(USBDeviceRequest)&deviceRequest,
 													this,
 													kIOAudioControlUsageOutput,
-													'subM');
+													kIOAudioToggleControlSubTypeLFEMute);
 	FailIf (NULL == muteControl, Exit);
 
 	audioEngine->addDefaultAudioControl (muteControl);
-	muteControl->release ();
 
 	audioEngine->completeConfigurationChange ();
 	audioEngine->resumeAudioEngine ();
@@ -229,7 +154,7 @@ bool AppleiSubEngine::handleOpen (IOService * forClient, IOOptionBits options, v
 	resultCode = TRUE;
 
 Exit:
-	debug5IOLog ("-AppleiSubEngine[%p]::handleOpen (%p, 0x%lx, %p)\n", this, forClient, options, arg);
+	debug4IOLog ("-AppleiSubEngine[%p]::handleOpen (%p, 0x%lx, %p) = %d\n", this, forClient, resultCode);
 	return resultCode;
 }
 
@@ -243,40 +168,29 @@ bool AppleiSubEngine::init (OSDictionary * properties) {
 
 	resultCode = TRUE;
 
+	// aml 2.28.02 initialize member vars
+	mFormat.altInterface = kDefaultiSubAltInterface;		
+	mFormat.numChannels = kDefaultNumChannels;		
+	mFormat.bytesPerSample = kDefaultBytesPerSample;		
+	mFormat.outputSampleRate = kDefaultOutputSampleRate;		
+
 Exit:
 	debug4IOLog ("-AppleiSubEngine[%p]::init (%p) = %d\n", this, properties, resultCode);
 	return resultCode;
 }
 
-IOReturn AppleiSubEngine::message (UInt32 type, IOService * provider, void * arg) {
-    IOReturn						resultCode;
-
-	debug4IOLog ("+AppleiSubEngine[%p]::message (0x%x, %p)\n", this, type, provider);
-
-	resultCode = kIOReturnSuccess;
-
-	switch (type) {
-		case kIOMessageServiceIsRequestingClose:
-			debugIOLog ("kIOMessageServiceIsRequestingClose\n");
-		case kIOMessageServiceIsTerminated:
-			if (kIOMessageServiceIsRequestingClose != type) debugIOLog ("kIOMessageServiceIsTerminated\n");
-			if ((NULL != streamInterface) && (streamInterface == provider)) {
-				debugIOLog ("stopping iSub\n");
-				shouldStop = 1;
-				streamInterface->close (this);
-				streamInterface = NULL;
-			}
-			break;
-		default:
-			;
-	}
-
-	debug4IOLog ("-AppleiSubEngine[%p]::message (0x%x, %p)\n", this, type, provider);
-	return resultCode;
-}
-
 bool AppleiSubEngine::start (IOService * provider) {
 	Boolean							resultBool;
+	IOUSBFindInterfaceRequest		findRequest;
+	IOUSBFindEndpointRequest		audioIsochEndpoint;
+	IOReturn						resultIOReturn;
+	UInt32							i;
+	UInt32							frameListNum;
+	UInt16							numQueued;
+	static IOPMPowerState			iSubPowerStates[2] = {
+		{kIOPMPowerStateVersion1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
+		{kIOPMPowerStateVersion1, kIOPMDeviceUsable, kIOPMPowerOn, kIOPMPowerOn, 0, 0, 0, 0, 0, 0, 0, 0}
+	};
 
 	debug3IOLog ("+AppleiSubEngine[%p]::start (%p)\n", this, provider);
 
@@ -288,10 +202,85 @@ bool AppleiSubEngine::start (IOService * provider) {
 
 	ourInterfaceNumber = streamInterface->GetInterfaceNumber ();
 	debug2IOLog ("AppleiSubEngine->ourInterfaceNumber = %d\n", ourInterfaceNumber);
-	alternateInterfaceID = 4;
+	// aml 1.18.02 replaced constant with member
+	alternateInterfaceID = mFormat.altInterface;		
 
-	interfaceLock = IORecursiveLockAlloc ();
-	FailIf (NULL == interfaceLock, Exit);
+	PMinit ();
+	streamInterface->joinPMtree (this);
+	if (pm_vars != NULL) {
+		registerPowerDriver (this, iSubPowerStates, NUM_POWER_STATES);
+		changePowerStateTo (1);
+	}
+
+	// aml 1.18.02 replaced sample rate with member
+	// aml 2.13.02 replaced kInputSampleRate with mOutputSampleRate
+	// aml 2.13.02 added num channels and bit depth constants, instead of '4'
+	frameListSize = CalculateNumSamplesPerBuffer (mFormat.outputSampleRate, NUM_ISUB_FRAMES_PER_LIST) * mFormat.numChannels * mFormat.bytesPerSample;
+	debug4IOLog ("format = %ldHz, %ld channels, %ld bits\n", mFormat.outputSampleRate, mFormat.numChannels, mFormat.bytesPerSample * 8);
+#if AML_DEBUGLOG
+	IOLog ("AppleiSubEngine::handleOpen: frameListSize = %d\n", frameListSize);
+#endif
+	debug2IOLog ("frameListSize = %d\n", frameListSize);
+	// aml 2.13.02 replaced kInputSampleRate with mOutputSampleRate
+	// aml 2.13.02 added num channels and bit depth members, instead of '4'
+	bufferSize = CalculateNumSamplesPerBuffer (mFormat.outputSampleRate, NUM_ISUB_FRAMES_PER_LIST, NUM_ISUB_FRAME_LISTS) * mFormat.numChannels * mFormat.bytesPerSample;
+	debug2IOLog ("bufferSize = %d\n", bufferSize);
+	sampleBuffer = IOMallocAligned (round_page (bufferSize), PAGE_SIZE);
+	FailIf (NULL == sampleBuffer, Exit);
+	// aml 2.13.02 changed from clearing by longs to words
+	for (i = 0; i < bufferSize / sizeof(UInt16); i++) {
+		((UInt16*)sampleBuffer)[i] = 0;
+	}
+	sampleBufferDescriptor = IOMemoryDescriptor::withAddress (sampleBuffer, bufferSize, kIODirectionNone);
+	FailIf (NULL == sampleBufferDescriptor, Exit);
+
+	FailIf (kIOReturnSuccess != PrepareFrameLists (frameListSize), Exit);
+//	FailIf (TRUE == streamOpened, Exit);
+//	IOLog ("after TRUE == streamOpened\n");
+	shouldCloseStream = FALSE;
+	FailIf (FALSE == streamInterface->open (this), Exit);
+	streamOpened = TRUE;
+
+	resultIOReturn = streamInterface->SetAlternateInterface (this, kRootAlternateSetting);
+	FailIf (kIOReturnSuccess != resultIOReturn, Exit);
+
+	// aml 1.18.02 replaced 4 (16 bit stereo alternate interface) with member
+ 	resultIOReturn = streamInterface->SetAlternateInterface (this, mFormat.altInterface);
+	FailIf (kIOReturnSuccess != resultIOReturn, Exit);
+
+	// Acquire a PIPE for the isochronous stream.
+	audioIsochEndpoint.type = kUSBIsoc;
+	audioIsochEndpoint.direction = kUSBOut;
+
+	thePipe = streamInterface->FindNextPipe (NULL, &audioIsochEndpoint);
+	FailIf (NULL == thePipe, Exit);
+	thePipe->retain ();
+
+	// The iSub is a little funky and reseting it improves (but doesn't totally eliminate) funky behavior.
+//	streamInterface->GetDevice()->ResetDevice ();
+
+	// Start the iSub playing silence now
+	loopCount = 0;
+
+	numUSBFrameListsNotOutstanding = 0;
+	numQueued = 0;
+	currentFrameList = 0;
+	currentByteOffset = 0;
+	theFirstFrame = streamInterface->GetDevice()->GetBus()->GetFrameNumber () + kMinimumiSubFrameOffset;
+	for (frameListNum = currentFrameList; numQueued < NUM_ISUB_FRAME_LISTS_TO_QUEUE; frameListNum++) {
+		resultIOReturn = WriteFrameList (frameListNum);
+		FailIf (kIOReturnSuccess != resultIOReturn, Exit);
+		numQueued++;
+	}
+	debug2IOLog ("writeFrameList resultIOReturn = 0x%lx\n", resultIOReturn);
+
+	// Find control interface on iSub device
+	findRequest.bInterfaceClass = 1;
+	findRequest.bInterfaceSubClass = 1;
+	findRequest.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
+	findRequest.bAlternateSetting = kIOUSBFindInterfaceDontCare;
+	controlInterface = streamInterface->GetDevice()->FindNextInterface (NULL, &findRequest);
+	FailIf (NULL == controlInterface, Exit);
 
 	registerService ();
 	resultBool = TRUE;
@@ -301,27 +290,69 @@ Exit:
 	return resultBool;
 }
 
-void AppleiSubEngine::stop (IOService * provider) {
-	debug2IOLog("+AppleiSubEngine[%p]::stop ()\n", this);
+bool AppleiSubEngine::willTerminate (IOService * provider, IOOptionBits options) {
+	bool					resultCode;
 
-	// We've been asked to go away, so we should really stop now
-	shouldStop = 1;
+	debug5IOLog ("+AppleiSubEngine[%p]::willTerminate (%p, 0x%x) rc =\n", this, provider, options, getRetainCount ());
 
-	super::stop (provider);
+	if (FALSE == iSubUSBRunning) {
+		// close our stream interface and go away
+		if ((streamInterface != NULL) && (streamInterface == provider)) {
+			debugIOLog ("willTerminate closing iSub down\n");
+			if (NULL != thePipe) {
+				thePipe->release ();
+				thePipe = NULL;
+			}
+			streamInterface->close (this);
+			streamOpened = FALSE;
+		}
+	} else {
+		shouldCloseStream = TRUE;
+	}
 
-	debug2IOLog("-AppleiSubEngine[%p]::stop ()\n", this);
-	return;
+	resultCode = super::willTerminate (provider, options);
+
+	debug6IOLog ("-AppleiSubEngine[%p]::willTerminate (%p, 0x%x) = %d, rc=%d\n", this, provider, options, resultCode, getRetainCount ());
+	return resultCode;
 }
 
-bool AppleiSubEngine::terminate (IOOptionBits options) {
-	Boolean							resultCode;
+IOReturn AppleiSubEngine::setPowerState (unsigned long powerStateOrdinal, IOService *device) {
+	IOReturn					result;
 
-	debug4IOLog ("+AppleiSubEngine[%p]::terminate (0x%x), rc=%d\n", this, options, getRetainCount ());
+	debug4IOLog("AppleiSubEngine[%p]::setPowerState (%lu, %p)\n", this, powerStateOrdinal, device);
 
-	resultCode = super::terminate (options);
+	if (device == this) {
+		switch (powerStateOrdinal) {
+			case 0:		// Power off state
+				debugIOLog ("sleeping...\n");
+				sleeping = TRUE;
+				StopiSub ();
+				result = IOPMNoErr;
+				break;
+			case 1:		// Power on state
+				debugIOLog ("waking...\n");
+				sleeping = FALSE;
+				// Don't do the reset on this thread because it takes too long.
+/*				if (NULL != streamInterface) {
+					streamInterface->GetDevice()->ResetDevice ();
+					// The ResetDevice causes the iSub to loose its volume and mute states, so put them back.
+					if (NULL != muteControl) {
+						muteControl->flushValue ();
+					}
+	
+					if (NULL != iSubVolumeControl) {
+						iSubVolumeControl->flushValue ();
+					}
+				} */
+	
+				result = IOPMNoErr;
+				break;
+			default:
+				result = IOPMNoSuchState;
+		}
+	}
 
-	debug5IOLog ("-AppleiSubEngine[%p]::terminate (0x%x) = %d, rc=%d\n", this, options, resultCode, getRetainCount ());
-	return resultCode;
+	return result;
 }
 
 #pragma mark -iSub Routines-
@@ -367,16 +398,10 @@ IOReturn AppleiSubEngine::deviceRequest (IOUSBDevRequest * request, AppleiSubEng
 	debug4IOLog ("+AppleiSubEngine[%p]::deviceRequest (%p, %p)\n", self, request, completion);
 	result = kIOReturnSuccess;
 	if (self->streamInterface && request) {
-            FailIf (NULL == self->interfaceLock, Exit);
-
-            IORecursiveLockLock (self->interfaceLock);
-            result = self->streamInterface->DeviceRequest (request, completion);
-            IORecursiveLockUnlock (self->interfaceLock);
+		result = self->streamInterface->DeviceRequest (request, completion);
     }
 
 	debug4IOLog ("-AppleiSubEngine[%p]::deviceRequest (%p, %p)\n", self, request, completion);
-
-Exit:
 	return result;
 }
 
@@ -384,10 +409,6 @@ UInt32 AppleiSubEngine::GetCurrentByteCount (void) {
 	return currentByteOffset;
 }
 
-UInt16 AppleiSubEngine::GetCurrentFrameList (void) {
-	return currentFrameList;
-}
-
 UInt32 AppleiSubEngine::GetCurrentLoopCount (void) {
 	return loopCount;
 }
@@ -396,10 +417,6 @@ IOMemoryDescriptor * AppleiSubEngine::GetSampleBuffer (void) {
 	return sampleBufferDescriptor;
 }
 
-volatile AbsoluteTime * AppleiSubEngine::GetLoopTime (void) {
-	return &lastLoopTime;
-}
-
 IOReturn AppleiSubEngine::PrepareFrameLists (UInt32 frameListSize) {
 	IOReturn						result;
 	UInt32							frameIndex;
@@ -422,62 +439,47 @@ Exit:
 }
 
 IOReturn AppleiSubEngine::StartiSub (void) {
-    AbsoluteTime					currentTime;
-	AbsoluteTime					initialTimestampOffset;
 	UInt32							frameListNum;
-	UInt16							numQueued;
+	UInt16							numQueued = 0;	// aml this was unititialized and the for loop below never ran sometimes!
     IOReturn						resultCode;
 
 	debug2IOLog ("+AppleiSubEngine[%p]::StartiSub ()\n", this);
 
+	FailWithAction (TRUE == sleeping, resultCode = kIOReturnOffline, Exit);
 	resultCode = kIOReturnError;
-#if !CONTINUOUS_STREAMING
-	loopCount = 0;
-	nanoseconds_to_absolutetime ((kMinimumiSubFrameOffset) * 1000 * 1000, &initialTimestampOffset);
-	clock_get_uptime (&currentTime);
-	ADD_ABSOLUTETIME (&currentTime, &initialTimestampOffset);
-	lastLoopTime.hi = currentTime.hi;
-	lastLoopTime.lo = currentTime.lo;
-
-	shouldStop = 0;
-	numQueued = 0;
-	currentFrameList = 0;
-	currentByteOffset = 0;
-	theFirstFrame = streamInterface->GetDevice()->GetBus()->GetFrameNumber () + kMinimumiSubFrameOffset;
-	for (frameListNum = currentFrameList; numQueued < NUM_ISUB_FRAME_LISTS_TO_QUEUE; frameListNum++) {
-		resultCode = WriteFrameList (frameListNum);
-		FailIf (kIOReturnSuccess != resultCode, Exit);
-		numQueued++;
-	}
-#else
 	iSubRunning = TRUE;
+
+#if ABORT_PIPE_ON_START
+	FailIf (NULL == thePipe, Exit);
+	thePipe->Abort ();		// let's kill all outstanding IO and start right back at the beginning
+	iSubUSBRunning = FALSE;
+//	FailIf (NULL == streamInterface, Exit);
+//	theFirstFrame = streamInterface->GetDevice()->GetBus()->GetFrameNumber () + kMinimumiSubFrameOffset;
+//	for (frameListNum = currentFrameList; numQueued < NUM_ISUB_FRAME_LISTS_TO_QUEUE; frameListNum++) {
+//		resultCode = WriteFrameList (frameListNum);
+//		FailIf (kIOReturnSuccess != resultCode, Exit);
+//		numQueued++;
+//	}
+#else
 	loopCount = 0xFFFFFFFF;			// so that it will go to 0 in the write completion routine when the frames are aborted
-//	clock_get_uptime (&currentTime);
-//	lastLoopTime.hi = currentTime.hi;
-//	lastLoopTime.lo = currentTime.lo;
 	currentFrameList = NUM_ISUB_FRAME_LISTS - 1;
 //	theFirstFrame = 0;		// force the completion routine to calculate the correct starting frame
 //	FailIf (NULL == thePipe, Exit);
 //	thePipe->Abort ();		// let's kill all outstanding IO and start right back at the beginning
+//	iSubUSBRunning = FALSE;
 //	currentByteOffset = 0;
+#endif // ABORT_PIPE_ON_START
+
 	resultCode = kIOReturnSuccess;
+
 	if (FALSE == iSubUSBRunning) {
-		// Why aren't the USB writes running?  They should be always running...
-#if DEBUGLOG
-		IOLog ("!!!iSub USB transport isn't running!!!\n");
-#endif
-
 		loopCount = 0;
-		nanoseconds_to_absolutetime ((kMinimumiSubFrameOffset) * 1000 * 1000, &initialTimestampOffset);
-		clock_get_uptime (&currentTime);
-		ADD_ABSOLUTETIME (&currentTime, &initialTimestampOffset);
-		lastLoopTime.hi = currentTime.hi;
-		lastLoopTime.lo = currentTime.lo;
 
-		shouldStop = 0;
+		numUSBFrameListsNotOutstanding = 0;
 		numQueued = 0;
 		currentFrameList = 0;
 		currentByteOffset = 0;
+		FailIf (NULL == streamInterface, Exit);
 		theFirstFrame = streamInterface->GetDevice()->GetBus()->GetFrameNumber () + kMinimumiSubFrameOffset;
 		for (frameListNum = currentFrameList; numQueued < NUM_ISUB_FRAME_LISTS_TO_QUEUE; frameListNum++) {
 			resultCode = WriteFrameList (frameListNum);
@@ -485,7 +487,6 @@ IOReturn AppleiSubEngine::StartiSub (void) {
 			numQueued++;
 		}
 	}
-#endif
 
 Exit:
 	debug3IOLog ("-AppleiSubEngine[%p]::StartiSub (), result = %d\n", this, resultCode);
@@ -498,9 +499,6 @@ IOReturn AppleiSubEngine::StopiSub (void) {
 	debug2IOLog ("+AppleiSubEngine[%p]::StopiSub ()\n", this);
 
 	iSubRunning = FALSE;
-#if !CONTINUOUS_STREAMING
-	shouldStop = 1;
-#endif
 
 	debug2IOLog ("-AppleiSubEngine[%p]::StopiSub ()\n", this);
 
@@ -520,9 +518,11 @@ IOReturn AppleiSubEngine::WriteFrameList (UInt32 frameListNum) {
 
 	firstFrame = (frameListNum % NUM_ISUB_FRAME_LISTS_TO_QUEUE) * NUM_ISUB_FRAMES_PER_LIST;
 
-	CalculateSamplesPerFrame (44100, &averageFrameSamples, &additionalSampleFrameFreq);
-	averageFrameSize = averageFrameSamples * 4;
-	alternateFrameSize = (averageFrameSamples + 1) * 4;
+	// aml 2.13.02 replaced kInputSampleRate with kOutputSampleRate
+	CalculateSamplesPerFrame (mFormat.outputSampleRate, &averageFrameSamples, &additionalSampleFrameFreq);
+	// aml 2.13.02 added num channels and bit depth constants, instead of '4'
+	averageFrameSize = averageFrameSamples * mFormat.numChannels * mFormat.bytesPerSample;
+	alternateFrameSize = (averageFrameSamples + 1) * mFormat.numChannels * mFormat.bytesPerSample;
 
 	if (additionalSampleFrameFreq) {
 		for (frameIndex = 0; frameIndex < NUM_ISUB_FRAMES_PER_LIST; frameIndex++) {
@@ -542,43 +542,33 @@ IOReturn AppleiSubEngine::WriteFrameList (UInt32 frameListNum) {
 		}
 	}
 
-	retain ();		// Don't want the driver being terminated until our completion routine runs.
-	result = thePipe->Write (soundBuffer[frameListNum], theFirstFrame, NUM_ISUB_FRAMES_PER_LIST, &theFrames[firstFrame], &usbCompletion[frameListNum]);
+    FailIf (NULL == thePipe, Exit);
+//	retain ();		// Don't want the driver being terminated until our completion routine runs.
+    result = thePipe->Write (soundBuffer[frameListNum], theFirstFrame, NUM_ISUB_FRAMES_PER_LIST, &theFrames[firstFrame], &usbCompletion[frameListNum]);
 
 	if (result != kIOReturnSuccess) {
-		debug6IOLog ("++AppleiSubEngine[%p]::WriteFrameList (%d) - error writing to pipe at frame %lu - current = %lu: 0x%x\n", this, frameListNum, (UInt32)theFirstFrame, (UInt32)streamInterface->GetDevice()->GetBus()->GetFrameNumber(), result);
+		if (streamInterface) {
+			debug6IOLog ("++AppleiSubEngine[%p]::WriteFrameList (%d) - error writing to pipe at frame %lu - current = %lu: 0x%x\n", this, frameListNum, (UInt32)theFirstFrame, (UInt32)streamInterface->GetDevice()->GetBus()->GetFrameNumber(), result);
+		}
 		iSubUSBRunning = FALSE;
 	} else {
 		theFirstFrame += NUM_ISUB_FRAMES_PER_LIST;
 		iSubUSBRunning = TRUE;
 	}
 
+Exit:
 	return result;
 }
 
-void AppleiSubEngine::WriteHandler (AppleiSubEngine * self, UInt32 * buffer, IOReturn result, IOUSBIsocFrame * pFrames) {
-    AbsoluteTime					currentTime;
+void AppleiSubEngine::WriteHandler (void * object, void * buffer, IOReturn result, IOUSBIsocFrame * pFrames) {
+	AppleiSubEngine *				self;
 	UInt64							currentUSBFrame;
-//	static UInt64					lastScheduledUSBFrame = 0;
 	UInt32							frameListToWrite;
 	UInt32							i;
-#if DEBUGLOG
-	UInt32							delta;
-	static AbsoluteTime				lastTime = {0, 0};
-	AbsoluteTime					diff;
-	UInt64							nanos;
-#endif
-#if DEBUGTIMESTAMPS
-	static AbsoluteTime				lastLoopTime = {0, 0};
-	AbsoluteTime					diff;
-	UInt64							nanos;
-#endif
 
+	self = (AppleiSubEngine *)object;
 
-	// Zero the data in the buffer so that this buffer just contains silence
-	for (i = 0; i < self->frameListSize / 4; i++) {
-		buffer[i] = 0;
-	}
+	FailIf (TRUE == self->sleeping, Exit);
 
 	if (result != kIOReturnSuccess) {
 #if DEBUGLOG
@@ -587,8 +577,18 @@ void AppleiSubEngine::WriteHandler (AppleiSubEngine * self, UInt32 * buffer, IOR
 		FailIf (NULL == self->streamInterface, Exit);
 		currentUSBFrame = self->streamInterface->GetDevice()->GetBus()->GetFrameNumber ();
 		switch (result) {
+#if ABORT_PIPE_ON_START
+			case kIOReturnAborted:
+#if AML_DEBUGLOG
+				IOLog ("AppleiSubEngine::WriteHandler() aborted.\n");
+#endif
+				goto Exit;			// do nothing
+				break;
+			case kIOReturnOverrun:
+#else
 			case kIOReturnOverrun:
 			case kIOReturnAborted:
+#endif
 			default:
 				// skip ahead and see if that helps
 				if (self->theFirstFrame <= currentUSBFrame) {
@@ -601,68 +601,28 @@ void AppleiSubEngine::WriteHandler (AppleiSubEngine * self, UInt32 * buffer, IOR
 		}
 	}
 
+	// aml 4.25.02 moved below error checking!
+
+	// Zero the data in the buffer so that this buffer just contains silence
+	// aml 2.13.02 changed from clearing by longs to words
+	for (i = 0; i < self->frameListSize / sizeof(UInt16); i++) {
+		((UInt16*)buffer)[i] = 0;
+	}
+
 	if ((NUM_ISUB_FRAME_LISTS - 1) == self->currentFrameList) {
-		clock_get_uptime (&currentTime);
-#if DEBUGLOG
-		diff.hi = currentTime.hi;
-		diff.lo = currentTime.lo;
-		SUB_ABSOLUTETIME (&diff, &lastTime);
-		lastTime.hi = currentTime.hi;
-		lastTime.lo = currentTime.lo;
-		absolutetime_to_nanoseconds (diff, &nanos);
-		delta = (UInt32)(nanos / (1000 * 1000));
-		if (delta > 10) {
-			if (delta - 10 > 2)
-				IOLog ("late = %ld\n", delta - 10);
-		} else {
-			if (10 - delta > 2)
-				IOLog ("early = %ld\n", 10 - delta);
-		}
-#endif
-#if DEBUGTIMESTAMPS
-		diff.hi = currentTime.hi;
-		diff.lo = currentTime.lo;
-		SUB_ABSOLUTETIME (&diff, &lastLoopTime);
-		lastLoopTime.hi = currentTime.hi;
-		lastLoopTime.lo = currentTime.lo;
-		absolutetime_to_nanoseconds (diff, &nanos);
-		IOLog ("delta = %ld\n", (UInt32)(nanos / (1000 * 1000)));
-#endif
-		self->lastLoopTime.hi = currentTime.hi;
-		self->lastLoopTime.lo = currentTime.lo;
 		if (TRUE == self->iSubRunning) {
 			self->loopCount++;
 			self->currentByteOffset = 0;
 		}
 		self->currentFrameList = 0;
 	} else {
-#if DEBUGLOG
-		clock_get_uptime (&currentTime);
-		diff.hi = currentTime.hi;
-		diff.lo = currentTime.lo;
-		SUB_ABSOLUTETIME (&diff, &lastTime);
-		lastTime.hi = currentTime.hi;
-		lastTime.lo = currentTime.lo;
-		absolutetime_to_nanoseconds (diff, &nanos);
-		delta = (UInt32)(nanos / (1000 * 1000));
-		if (delta > 10) {
-			if (delta - 10 > 2)
-				IOLog ("late = %ld\n", delta - 10);
-		} else {
-			if (10 - delta > 2)
-				IOLog ("early = %ld\n", 10 - delta);
-		}
-#endif
 		self->currentFrameList++;
 		if (TRUE == self->iSubRunning) {
 			self->currentByteOffset = self->currentFrameList * self->frameListSize;
 		}
 	}
 
-	if (self->shouldStop > 0) {
-		debug3IOLog ("++AppleiSubEngine[%p]::WriteHandler () - stopping: %d\n", self, self->shouldStop);
-		self->shouldStop++;
-	} else {
+	if (FALSE == self->shouldCloseStream) {
 		frameListToWrite = self->currentFrameList + NUM_ISUB_FRAME_LISTS_TO_QUEUE - 1;
 		if (frameListToWrite >= NUM_ISUB_FRAME_LISTS) {
 			frameListToWrite -= NUM_ISUB_FRAME_LISTS;
@@ -671,6 +631,25 @@ void AppleiSubEngine::WriteHandler (AppleiSubEngine * self, UInt32 * buffer, IOR
 	}
 
 Exit:
-	self->release ();
+	if (TRUE == self->shouldCloseStream) {
+		#if DEBUGLOG
+		IOLog ("++AppleiSubEngine[%p]::WriteHandler () - stopping: %d\n", self, self->numUSBFrameListsNotOutstanding);
+		#endif
+		self->numUSBFrameListsNotOutstanding++;
+		if (self->numUSBFrameListsNotOutstanding == NUM_ISUB_FRAME_LISTS_TO_QUEUE) {
+			#if DEBUGLOG
+			IOLog ("iSub last write completed, closing streamInterface\n");
+			#endif
+			if (NULL != self->thePipe) {
+				self->thePipe->release ();
+				self->thePipe = NULL;
+			}
+
+			self->streamInterface->close (self);
+			self->streamOpened = FALSE;
+			self->iSubUSBRunning = FALSE;
+		}
+	}
+//	self->release ();
 	return;
 }
